@@ -1,44 +1,27 @@
-from base64 import b64encode
-from datetime import (
-    datetime,
-    timedelta,
-    )
-from hashlib import md5
 import re
 import struct
 import zlib
-try:
-    import simplejson as json
-except ImportError:
-    import json
+from base64 import b64encode
+from datetime import datetime, timedelta
+from hashlib import md5
 
 from webob.byterange import ContentRange
-
-from webob.cachecontrol import (
-    CacheControl,
-    serialize_cache_control,
-    )
-
+from webob.cachecontrol import CacheControl, serialize_cache_control
 from webob.compat import (
-    PY3,
+    PY2,
     bytes_,
     native_,
+    string_types,
     text_type,
     url_quote,
     urlparse,
-    )
-
-from webob.cookies import (
-    Cookie,
-    make_cookie,
-    )
-
+)
+from webob.cookies import Cookie, make_cookie
 from webob.datetime_utils import (
     parse_date_delta,
     serialize_date_delta,
     timedelta_to_seconds,
-    )
-
+)
 from webob.descriptors import (
     CHARSET_RE,
     SCHEME_RE,
@@ -55,11 +38,15 @@ from webob.descriptors import (
     serialize_content_range,
     serialize_etag_response,
     serialize_int,
-    )
-
+)
 from webob.headers import ResponseHeaders
 from webob.request import BaseRequest
-from webob.util import status_reasons, status_generic_reasons, warn_deprecation
+from webob.util import status_generic_reasons, status_reasons, warn_deprecation
+
+try:
+    import simplejson as json
+except ImportError:
+    import json
 
 __all__ = ['Response']
 
@@ -150,14 +137,14 @@ class Response(object):
       :attr:`~Response.text` will raise an error if it fails to decode the
       :attr:`~Response.body`.
 
-    * ``default_conditional_response`` is set to False. This flag may be set to
-      True so that all ``Response`` objects will attempt to check the original
-      request for conditional response headers. See
+    * ``default_conditional_response`` is set to ``False``. This flag may be
+      set to ``True`` so that all ``Response`` objects will attempt to check
+      the original request for conditional response headers. See
       :meth:`~Response.conditional_response_app` for more information.
 
-    * ``default_body_encoding`` is set to 'UTF-8' by default, it exists to
-      allow users to get/set the Response object using .text, even if no
-      charset has been set for the Content-Type.
+    * ``default_body_encoding`` is set to 'UTF-8' by default. It exists to
+      allow users to get/set the ``Response`` object using ``.text``, even if
+      no ``charset`` has been set for the ``Content-Type``.
     """
 
     default_content_type = 'text/html'
@@ -343,7 +330,7 @@ class Response(object):
 
         This reads the response as represented by ``str(resp)``; it
         may not read every valid HTTP response properly.  Responses
-        must have a ``Content-Length``"""
+        must have a ``Content-Length``."""
         headerlist = []
         status = fp.readline().strip()
         is_text = isinstance(status, text_type)
@@ -386,7 +373,7 @@ class Response(object):
         return r
 
     def copy(self):
-        """Makes a copy of the response"""
+        """Makes a copy of the response."""
         # we need to do this for app_iter to be reusable
         app_iter = list(self._app_iter)
         iter_close(self._app_iter)
@@ -413,7 +400,7 @@ class Response(object):
             self.body
         parts += map('%s: %s'.__mod__, self.headerlist)
         if not skip_body and self.body:
-            parts += ['', self.text if PY3 else self.body]
+            parts += ['', self.body if PY2 else self.text]
         return '\r\n'.join(parts)
 
     #
@@ -422,7 +409,7 @@ class Response(object):
 
     def _status__get(self):
         """
-        The status string
+        The status string.
         """
         return self._status
 
@@ -434,7 +421,7 @@ class Response(object):
         else:
             self.status_code = code
             return
-        if PY3:
+        if not PY2:
             if isinstance(value, bytes):
                 value = value.decode('ascii')
         elif isinstance(value, text_type):
@@ -458,7 +445,7 @@ class Response(object):
 
     def _status_code__get(self):
         """
-        The status as an integer
+        The status as an integer.
         """
         return int(self._status.split()[0])
 
@@ -477,7 +464,7 @@ class Response(object):
 
     def _headerlist__get(self):
         """
-        The list of response headers
+        The list of response headers.
         """
         return self._headerlist
 
@@ -497,7 +484,7 @@ class Response(object):
 
     def _headers__get(self):
         """
-        The headers in a dictionary-like object
+        The headers in a dictionary-like object.
         """
         if self._headers is None:
             self._headers = ResponseHeaders.view_list(self._headerlist)
@@ -573,7 +560,7 @@ class Response(object):
 
     def _json_body__get(self):
         """
-        Set/get the body of the response as JSON
+        Set/get the body of the response as JSON.
 
         .. note::
 
@@ -597,7 +584,7 @@ class Response(object):
     def _has_body__get(self):
         """
         Determine if the the response has a :attr:`~Response.body`. In
-        contrast to simply accessing :attr:`~Response.body` this method
+        contrast to simply accessing :attr:`~Response.body`, this method
         will **not** read the underlying :attr:`~Response.app_iter`.
         """
 
@@ -622,8 +609,8 @@ class Response(object):
 
     def _text__get(self):
         """
-        Get/set the text value of the body using the charset of the
-        Content-Type or the default_body_encoding.
+        Get/set the text value of the body using the ``charset`` of the
+        ``Content-Type`` or the ``default_body_encoding``.
         """
         if not self.charset and not self.default_body_encoding:
             raise AttributeError(
@@ -662,7 +649,7 @@ class Response(object):
     def _body_file__get(self):
         """
         A file-like object that can be used to write to the
-        body.  If you passed in a list app_iter, that app_iter will be
+        body.  If you passed in a list ``app_iter``, that ``app_iter`` will be
         modified by writes.
         """
         return ResponseBodyFile(self)
@@ -704,10 +691,10 @@ class Response(object):
 
     def _app_iter__get(self):
         """
-        Returns the app_iter of the response.
+        Returns the ``app_iter`` of the response.
 
-        If body was set, this will create an app_iter from that body
-        (a single-item list)
+        If ``body`` was set, this will create an ``app_iter`` from that
+        ``body`` (a single-item list).
         """
         return self._app_iter
 
@@ -715,7 +702,6 @@ class Response(object):
         if self._app_iter is not None:
             # Undo the automatically-set content-length
             self.content_length = None
-            self.content_md5 = None
         self._app_iter = value
 
     def _app_iter__del(self):
@@ -787,10 +773,10 @@ class Response(object):
 
     def _charset__get(self):
         """
-        Get/set the charset specified in Content-Type.
+        Get/set the ``charset`` specified in ``Content-Type``.
 
-        There is no checking to validate that a ``content_type`` actually allows
-        for a charset parameter.
+        There is no checking to validate that a ``content_type`` actually
+        allows for a ``charset`` parameter.
         """
         header = self.headers.get('Content-Type')
         if not header:
@@ -833,19 +819,20 @@ class Response(object):
 
     def _content_type__get(self):
         """
-        Get/set the Content-Type header. If no Content-Type header is set, this
-        will return None.
+        Get/set the ``Content-Type`` header. If no ``Content-Type`` header is
+        set, this will return ``None``.
 
         .. versionchanged:: 1.7
 
-            Setting a new Content-Type will remove all Content-Type parameters
-            and reset the charset to the default if the Content-Type is
-            ``text/*`` or XML (``application/xml``, or ``*/*+xml``)
+            Setting a new ``Content-Type`` will remove all ``Content-Type``
+            parameters and reset the ``charset`` to the default if the
+            ``Content-Type`` is ``text/*`` or XML (``application/xml`` or
+            ``*/*+xml``).
 
-            To preserve all Content-Type parameters you may use the following
-            code:
+            To preserve all ``Content-Type`` parameters, you may use the
+            following code:
 
-            .. code::
+            .. code-block:: python
 
                 resp = Response()
                 params = resp.content_type_params
@@ -862,6 +849,12 @@ class Response(object):
             self._content_type__del()
             return
         else:
+            if PY2 and isinstance(value, text_type):
+                value = value.encode("latin-1")
+
+            if not isinstance(value, string_types):
+                raise TypeError("content_type requires value to be of string_types")
+
             content_type = value
 
             # Set up the charset if the content-type doesn't have one
@@ -907,7 +900,7 @@ class Response(object):
         A dictionary of all the parameters in the content type.
 
         (This is not a view, set to change, modifications of the dict will not
-        be applied otherwise)
+        be applied otherwise.)
         """
         params = self.headers.get('Content-Type', '')
         if ';' not in params:
@@ -947,9 +940,10 @@ class Response(object):
     # set_cookie, unset_cookie, delete_cookie, merge_cookies
     #
 
-    def set_cookie(self, name=None, value='', max_age=None,
+    def set_cookie(self, name, value='', max_age=None,
                    path='/', domain=None, secure=False, httponly=False,
-                   comment=None, expires=None, overwrite=False):
+                   comment=None, expires=None, overwrite=False,
+                   samesite=None):
         """
         Set (add) a cookie for the response.
 
@@ -972,10 +966,10 @@ class Response(object):
            or ``None``. This value is used as the ``Max-Age`` of the generated
            cookie.  If ``expires`` is not passed and this value is not
            ``None``, the ``max_age`` value will also influence the ``Expires``
-           value of the cookie (``Expires`` will be set to now + max_age).  If
-           this value is ``None``, the cookie will not have a ``Max-Age`` value
-           (unless ``expires`` is set). If both ``max_age`` and ``expires`` are
-           set, this value takes precedence.
+           value of the cookie (``Expires`` will be set to ``now`` +
+           ``max_age``).  If this value is ``None``, the cookie will not have a
+           ``Max-Age`` value (unless ``expires`` is set). If both ``max_age``
+           and ``expires`` are set, this value takes precedence.
 
         ``path``
 
@@ -1000,6 +994,12 @@ class Response(object):
            in the cookie, if it's ``False``, the ``HttpOnly`` flag will not
            be sent in the cookie.
 
+        ``samesite``
+
+          A string representing the ``SameSite`` attribute of the cookie or
+          ``None``. If samesite is ``None`` no ``SameSite`` value will be sent
+          in the cookie. Should only be ``"strict"``, ``"lax"``, or ``"none"``.
+
         ``comment``
 
            A string representing the cookie ``Comment`` value, or ``None``.
@@ -1022,8 +1022,8 @@ class Response(object):
            local time are not supported. Timezone aware ``datetime.datetime``
            objects are converted to UTC.
 
-           This argument will be removed in future
-           versions of WebOb (version 1.9).
+           This argument will be removed in future versions of WebOb (version
+           1.9).
 
         ``overwrite``
 
@@ -1037,9 +1037,6 @@ class Response(object):
             warn_deprecation('Argument "expires" will be removed in a future '
                              'version of WebOb, please use "max_age".', 1.10, 1)
 
-        if name is None:
-            raise TypeError('set_cookie() takes at least 1 argument')
-
         if overwrite:
             self.unset_cookie(name, strict=False)
 
@@ -1058,27 +1055,24 @@ class Response(object):
 
         value = bytes_(value, 'utf-8')
 
-        cookie = make_cookie(
-            name, value, max_age=max_age, path=path,
-            domain=domain, secure=secure, httponly=httponly,
-            comment=comment)
-
+        cookie = make_cookie(name, value, max_age=max_age, path=path,
+                domain=domain, secure=secure, httponly=httponly,
+                comment=comment, samesite=samesite)
         self.headerlist.append(('Set-Cookie', cookie))
 
     def delete_cookie(self, name, path='/', domain=None):
         """
-        Delete a cookie from the client.  Note that path and domain must match
-        how the cookie was originally set.
+        Delete a cookie from the client.  Note that ``path`` and ``domain``
+        must match how the cookie was originally set.
 
-        This sets the cookie to the empty string, and max_age=0 so
+        This sets the cookie to the empty string, and ``max_age=0`` so
         that it should expire immediately.
         """
         self.set_cookie(name, None, path=path, domain=domain)
 
     def unset_cookie(self, name, strict=True):
         """
-        Unset a cookie with the given name (remove it from the
-        response).
+        Unset a cookie with the given name (remove it from the response).
         """
         existing = self.headers.getall('Set-Cookie')
         if not existing and not strict:
@@ -1098,9 +1092,9 @@ class Response(object):
 
     def merge_cookies(self, resp):
         """Merge the cookies that were set on this response with the
-        given `resp` object (which can be any WSGI application).
+        given ``resp`` object (which can be any WSGI application).
 
-        If the `resp` is a :class:`webob.Response` object, then the
+        If the ``resp`` is a :class:`webob.Response` object, then the
         other object will be modified in-place.
         """
         if not self.headers.get('Set-Cookie'):
@@ -1128,7 +1122,7 @@ class Response(object):
     def _cache_control__get(self):
         """
         Get/set/modify the Cache-Control header (`HTTP spec section 14.9
-        <http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9>`_)
+        <http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9>`_).
         """
         value = self.headers.get('cache-control', '')
         if self._cache_control_obj is None:
@@ -1182,7 +1176,7 @@ class Response(object):
         """
             Set expiration on this request.  This sets the response to
             expire in the given seconds, and any other attributes are used
-            for cache_control (e.g., private=True, etc).
+            for ``cache_control`` (e.g., ``private=True``).
         """
         if seconds is True:
             seconds = 0
@@ -1222,8 +1216,8 @@ class Response(object):
 
     def encode_content(self, encoding='gzip', lazy=False):
         """
-        Encode the content with the given encoding (only gzip and
-        identity are supported).
+        Encode the content with the given encoding (only ``gzip`` and
+        ``identity`` are supported).
         """
         assert encoding in ('identity', 'gzip'), \
             "Unknown encoding: %r" % encoding
@@ -1255,17 +1249,25 @@ class Response(object):
             self.content_encoding = None
             gzip_f.close()
         else:
-            # Weird feature: http://bugs.python.org/issue5784
-            self.body = zlib.decompress(self.body, -15)
+            try:
+                # RFC7230 section 4.2.2 specifies that the body should be wrapped
+                # inside a ZLIB (RFC1950) container ...
+                self.body = zlib.decompress(self.body)
+            except zlib.error:
+                # ... but there are nonconformant implementations around which send
+                # the data without the ZLIB container, so we use maximum window size
+                # decompression without header check (the - sign)
+                self.body = zlib.decompress(self.body, -15)
             self.content_encoding = None
 
     def md5_etag(self, body=None, set_content_md5=False):
         """
         Generate an etag for the response object using an MD5 hash of
-        the body (the body parameter, or ``self.body`` if not given)
+        the body (the ``body`` parameter, or ``self.body`` if not given).
 
-        Sets ``self.etag``
-        If ``set_content_md5`` is True sets ``self.content_md5`` as well
+        Sets ``self.etag``.
+
+        If ``set_content_md5`` is ``True``, sets ``self.content_md5`` as well.
         """
         if body is None:
             body = self.body
@@ -1317,11 +1319,14 @@ class Response(object):
 
     def conditional_response_app(self, environ, start_response):
         """
-        Like the normal __call__ interface, but checks conditional headers:
+        Like the normal ``__call__`` interface, but checks conditional headers:
 
-        * If-Modified-Since   (304 Not Modified; only on GET, HEAD)
-        * If-None-Match       (304 Not Modified; only on GET, HEAD)
-        * Range               (406 Partial Content; only on GET, HEAD)
+            * ``If-Modified-Since``   (``304 Not Modified``; only on ``GET``,
+              ``HEAD``)
+            * ``If-None-Match``       (``304 Not Modified``; only on ``GET``,
+              ``HEAD``)
+            * ``Range``               (``406 Partial Content``; only on ``GET``,
+              ``HEAD``)
         """
         req = BaseRequest(environ)
 
@@ -1383,7 +1388,7 @@ class Response(object):
 
     def app_iter_range(self, start, stop):
         """
-        Return a new app_iter built from the response app_iter, that
+        Return a new ``app_iter`` built from the response ``app_iter``, that
         serves up only the given ``start:stop`` range.
         """
         app_iter = self._app_iter
@@ -1424,7 +1429,7 @@ class ResponseBodyFile(object):
 
     def writelines(self, seq):
         """
-        Write a sequence of lines to the response
+        Write a sequence of lines to the response.
         """
         for item in seq:
             self.write(item)
@@ -1437,9 +1442,9 @@ class ResponseBodyFile(object):
 
     def tell(self):
         """
-        Provide the current location where we are going to start writing
+        Provide the current location where we are going to start writing.
         """
-        if self.response.app_iter is None: # pragma: no cover
+        if not self.response.has_body:
             return 0
 
         return sum([len(chunk) for chunk in self.response.app_iter])
@@ -1447,7 +1452,7 @@ class ResponseBodyFile(object):
 
 class AppIterRange(object):
     """
-    Wraps an app_iter, returning just a range of bytes
+    Wraps an ``app_iter``, returning just a range of bytes.
     """
 
     def __init__(self, app_iter, start, stop):
@@ -1502,10 +1507,11 @@ class AppIterRange(object):
 
 
 class EmptyResponse(object):
-    """An empty WSGI response.
+    """
+    An empty WSGI response.
 
     An iterator that immediately stops. Optionally provides a close
-    method to close an underlying app_iter it replaces.
+    method to close an underlying ``app_iter`` it replaces.
     """
 
     def __init__(self, app_iter=None):
@@ -1543,9 +1549,9 @@ def _content_type_has_charset(content_type):
     )
 
 def _request_uri(environ):
-    """Like wsgiref.url.request_uri, except eliminates :80 ports
+    """Like ``wsgiref.url.request_uri``, except eliminates ``:80`` ports.
 
-    Return the full request URI"""
+    Returns the full request URI."""
     url = environ['wsgi.url_scheme'] + '://'
 
     if environ.get('HTTP_HOST'):
@@ -1557,12 +1563,12 @@ def _request_uri(environ):
     elif url.endswith(':443') and environ['wsgi.url_scheme'] == 'https':
         url = url[:-4]
 
-    if PY3: # pragma: no cover
-        script_name = bytes_(environ.get('SCRIPT_NAME', '/'), 'latin-1')
-        path_info = bytes_(environ.get('PATH_INFO', ''), 'latin-1')
-    else:
+    if PY2:
         script_name = environ.get('SCRIPT_NAME', '/')
         path_info = environ.get('PATH_INFO', '')
+    else:
+        script_name = bytes_(environ.get('SCRIPT_NAME', '/'), 'latin-1')
+        path_info = bytes_(environ.get('PATH_INFO', ''), 'latin-1')
 
     url += url_quote(script_name)
     qpath_info = url_quote(path_info)
